/*
 * $Id: qctl.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: qctl.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <fcntl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <pwd.h>
#include <ufs/ufs/quota.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>

struct quotas {
	long id;
	int type;
	struct dqblk dqblk;
	char fsname[MAXPATHLEN + 1];
	char qfname[MAXPATHLEN + 1];
	struct quotas * next;
};

#define BUFLEN 4096

struct quotas * makequotas(char* file)
{
	FILE* fp;
	struct quotas *first=NULL,*current=NULL,*qptr;
	char string[BUFLEN],name[BUFLEN];
	long softblim,hardblim,softilim,hardilim,type,line=1;
	struct passwd *pw;

	if( !(fp=fopen(file,"r")) )
	{
		printf("Unable to open file %s with quotas specification.\n",file);
		exit(-1);
	}

	fgets(string,BUFLEN,fp);
	while( !feof(fp) )
	{
		if( !(qptr=(struct quotas *)malloc(sizeof(struct quotas))) )
		{
			printf("Unable to allocate memory.\n");
			exit(-1);
		}

		if( sscanf(string,"%ld %s %s %ld %ld %ld %ld",&type,name,qptr->fsname,
							 &softblim,&hardblim,&softilim,&hardilim) < 4 )
			printf("File %s, line %ld: bad line format. Skipped.\n",file,line);
		else
		{
			if( (pw=getpwnam(name)) == NULL )
				printf("File %s, line %ld: user %s is unknown.\n",file,line,name);
			else
			{
				if( !first )
					first=current=qptr;
				else
				{
					current->next=qptr;
					current=qptr;
				}
				current->next=NULL;
				current->id=pw->pw_uid;
				if( type == 0 )
				{
					current->type=USRQUOTA;
					if( (strlen(current->fsname)+11) > MAXPATHLEN )
					{
						printf("File %s, line %ld: filesystem name is too big.\n",
						file,line);
						exit(-1);
					}
					strcpy(current->qfname,current->fsname);
					strcat(current->qfname,"/quota.user");
				}
				else if( type == 1 )
				{
					current->type=GRPQUOTA;
					if( (strlen(current->fsname)+12) > MAXPATHLEN )
					{
						printf("File %s, line %ld: filesystem name is too big.\n",
						file,line);
						exit(-1);
					}
					strcpy(current->qfname,current->fsname);
					strcat(current->qfname,"/quota.group");
				}
				else
				{
					printf("File %s, line %ld: unknown quota type %ld."
								 "Cannot continue.\n",file,line,type);
					exit(-1);
				}
				current->dqblk.dqb_bhardlimit=hardblim;
				current->dqblk.dqb_bsoftlimit=softblim;
				current->dqblk.dqb_ihardlimit=hardilim;
				current->dqblk.dqb_isoftlimit=softilim;
				current->dqblk.dqb_btime=0;
				current->dqblk.dqb_itime=0;
			}
		}

		line++;
		fgets(string,BUFLEN,fp);
	}
	fclose(fp);

	return first;
}

void putquotas(struct quotas * quotas)
{
	int cmd,fd;
	register struct quotas* q;

	/* Traversing all quotas and trying to set them. */
	for(q=quotas;q;q=q->next)
		/* Trying system call. */
		if( quotactl(q->fsname,QCMD(Q_SETQUOTA,q->type),q->id,&(q->dqblk)) == 0 );
		/* Trying to set this quota manually. */
		else
		{
			printf("System call not succeeded. Using quota file %s.\n",q->qfname);
			if( (fd=open(q->qfname,O_WRONLY)) < 0 )
				printf("Error opening quota file %s.\n",q->qfname);
			else
			{
				lseek(fd,(long)q->id*sizeof(struct dqblk),0);
				if( write(fd,&(q->dqblk),sizeof(struct dqblk)) != sizeof(struct dqblk) )
					printf("Error writing to quota file %s.\n",q->qfname);
				close(fd);
			}
		}

	return;
}

int main(int argc,char* argv[])
{
	struct quotas *quotas;

	if( geteuid() != 0 )
	{
		printf("You must be root in order to run this utility.\n");
		exit(-1);
	}
	if( argc < 2 )
	{
		printf("No input file specified.\n");
		exit(-1);
	}
	if( (quotas=makequotas(argv[1])) == NULL )
		exit(-1);
	putquotas(quotas);

	return 0;
}
