/*
 * $Id: log.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: log.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <stdio.h>
#include <syslog.h>
#include <stdarg.h>
#include <unistd.h>

#define _LOG_SYSLOG 100
#define _LOG_STDERR 200

int _LogOpened=0;
int _LogType=-1;
int _LogLevel=0;

void LogOpenSyslog(const char* ident,const int facility)
/******************************************************************************
 * Tells message subsystem to use syslog.                                     *
 * Call:                                                                      *
 *  ident - program identification that will be used by syslog.               *
 *  facility - syslog facility to use.                                        *
 ******************************************************************************/
{
	#if 1
	openlog(ident,LOG_CONS|LOG_PID|LOG_NDELAY,facility);
	#endif
	_LogOpened=1;
	_LogType=_LOG_SYSLOG;
	_LogLevel=0;
	return;
}

void LogOpenStderr()
/******************************************************************************
 * Tells the message subsystem to use stderr.                                 *
 ******************************************************************************/
{
	_LogOpened=1;
	_LogType=_LOG_STDERR;
	_LogLevel=0;
	return;
}

void LogClose()
/******************************************************************************
 * Closes the message subsystem.                                              *
 ******************************************************************************/
{
	if( _LogType == _LOG_SYSLOG )
		closelog();
	_LogOpened=0;
	_LogType=-1;
	_LogLevel=0;
	return;
}

void SetLogLevel(const int level)
/******************************************************************************
 * Sets log level to level.                                                   *
 ******************************************************************************/
{
	_LogLevel=level;
	return;
}

void LogError(const int level,const char* message, ...)
/******************************************************************************
 * Handles error messages.                                                    *
 * Call:                                                                      *
 *  message - printf() style string.                                          *
 *  args - printf() style arguments.                                          *
 ******************************************************************************/
{
	va_list args;

	if( level > _LogLevel )
		return;
	va_start(args,message);
	if( !_LogOpened )
	{
		fprintf(stderr,"Pid %d: message subsystem isn't initialysed.\n",getpid());
		return;
	}
	if( _LogType == _LOG_SYSLOG )
		vsyslog(LOG_ERR,message,args);
	else if( _LogType == _LOG_STDERR )
	{
		fprintf(stderr,"ERR[%ld]: ",(long)getpid());
		vfprintf(stderr,message,args);
		fprintf(stderr,"\n");
	}
	else;
	va_end(args);

	return;
}

void LogDebug(const int level,const char* message, ...)
/******************************************************************************
 * Handles debug messages.                                                    *
 * Call:                                                                      *
 *  level - message log level.                                                *
 *  message - printf() style string.                                          *
 *  args - printf() style arguments.                                          *
 ******************************************************************************/
{
	va_list args;

	if( level > _LogLevel )
		return;
	va_start(args,message);
	if( !_LogOpened )
	{
		fprintf(stderr,"Pid %d: message subsystem isn't initialysed.\n",getpid());
		return;
	}
	if( _LogType == _LOG_SYSLOG )
		vsyslog(LOG_DEBUG,message,args);
	else if( _LogType == _LOG_STDERR )
	{
		fprintf(stderr,"DBG[%ld]: ",(long)getpid());
		vfprintf(stderr,message,args);
		fprintf(stderr,"\n");
	}
	else;
	va_end(args);

	return;
}

void LogInfo(const int level,const char* message, ...)
/******************************************************************************
 * Handles info.                                                              *
 * Call:                                                                      *
 *  message - printf() style string.                                          *
 *  args - printf() style arguments.                                          *
 ******************************************************************************/
{
	va_list args;

	if( level > _LogLevel )
		return;
	va_start(args,message);
	if( !_LogOpened )
	{
		fprintf(stderr,"Pid %d: message subsystem isn't initialysed.\n",getpid());
		return;
	}
	if( _LogType == _LOG_SYSLOG )
		vsyslog(LOG_INFO,message,args);
	else if( _LogType == _LOG_STDERR )
	{
		fprintf(stderr,"INFO[%ld]: ",(long)getpid());
		vfprintf(stderr,message,args);
		fprintf(stderr,"\n");
	}
	else;
	va_end(args);
	return;
}

void LogWarning(const int level,const char* message, ...)
/******************************************************************************
 * Handles warning message.                                                   *
 * Call:                                                                      *
 *  message - printf() style string.                                          *
 *  args - printf() style arguments.                                          *
 ******************************************************************************/
{
	va_list args;

	if( level > _LogLevel )
		return;
	va_start(args,message);
	if( !_LogOpened )
	{
		fprintf(stderr,"Pid %d: message subsystem isn't initialysed.\n",getpid());
		return;
	}
	if( _LogType == _LOG_SYSLOG )
		vsyslog(LOG_WARNING,message,args);
	else if( _LogType == _LOG_STDERR )
	{
		fprintf(stderr,"WARN[%ld]: ",(long)getpid());
		vfprintf(stderr,message,args);
		fprintf(stderr,"\n");
	}
	else;
	va_end(args);
	return;
}
