/*
 * $Id: rules.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: rules.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include "hash.h"
#include "rules.h"
#include "stringops.h"
#include "log.h"

#if 1
#define DEBUG
#endif

#define BUFLEN  2048

typedef struct RULE_CHAIN {
	RULE_PTR rule;
	struct RULE_CHAIN* next;
}RULE_CHAIN,*RULE_CHAIN_PTR;

RULE_PTR parse_rules(char* file,int* NRules)
/******************************************************************************
 * Parses mail rules from file and returns array of rules.                    *
 * Call:                                                                      *
 *  file - filename to read rules from.                                       *
 *  NRules - pointer to a variable where number of rules will be stored.      *
 * Return:                                                                    *
 *  NULL if an error occured, otherwise pointer to ruleset.                   *
 ******************************************************************************/
{
	int n,i;
	char string[BUFLEN];
	FILE* fp;
	RULE_PTR ruleset;
	RULE_CHAIN_PTR cur,root=NULL,temp;

	if( !(fp=fopen(file,"r")) )
	{
		printf("Unable to open file `%s' with mail filtering rules.\n",file);
		return NULL;
	}

	n=0;
	fgets(string,BUFLEN,fp);
	while( !feof(fp) )
	{
		if( !(temp=(RULE_CHAIN_PTR)malloc(sizeof(RULE_CHAIN))) ||
				!(temp->rule=(RULE_PTR)malloc(sizeof(RULE))) )
		{
			fclose(fp);
			printf("Unable to allocate memory for mail rule.\n");
			return NULL;
		}
		chomp(string);
		if( parse_rule(string,temp->rule) )
			return NULL;
		if( !root )
		{
			root=temp;
			cur=temp;
		}
		else
		{
			cur->next=temp;
			cur=cur->next;
		}
		cur->next=NULL;
		n++;
			
		fgets(string,BUFLEN,fp);
	}

	if( !(ruleset=(RULE_PTR)calloc(n,sizeof(RULE))) )
	{
		printf("Unable to allocate memory for mail rules.\n");
		return NULL;
	}

	cur=root;
	i=0;
	while( cur )
	{
		bcopy(cur->rule,ruleset+i,sizeof(RULE));
		temp=cur;
		cur=cur->next;
		free(temp->rule);
		free(temp);
		i++;
	}

	fclose(fp);
	*NRules=n;
	return ruleset;
}

#define ISSPACE(c) ( (c) == ' ' || (c) == '\t' )

int parse_rule(char* string,RULE_PTR rule)
/******************************************************************************
 * Parses one rule from string.                                               *
 ******************************************************************************/
{
	int i,j;

	if( string == NULL || rule == NULL )
	{
		printf("%s,%d: NULL pointer passed to parse_rule().\n",__FILE__,__LINE__);
		exit(1);
	}
	i=0;
	/* Skipping spaces. */
	while( ISSPACE(string[i]) )
		i++;

	/* First word. */
	j=0;
	while( !ISSPACE(string[i+j]) )
		j++;
	if (j != 4 ||
	    (strncasecmp("pass", string + i, 4) &&
	     strncasecmp("drop", string + i, 4))) {
		printf("Error in rule `%s':\n"
		    "first word in rule must be `pass' or `drop'.\n", string);
		return 1;
	}
	if (!strncasecmp("pass", string, 4)) {
		#ifdef DEBUG
		printf("Pass rule.\n");
		#endif
		rule->rule_type=PASS_RULE;
	} else {
		#ifdef DEBUG
		printf("Drop rule.\n");
		#endif
		rule->rule_type=DROP_RULE;
	}
	i+=j;
	/* Skipping spaces. */
	while( ISSPACE(string[i]) )
		i++;
	/* Second word must be `from'. */
	j=0;
	while( !ISSPACE(string[i+j]) )
		j++;
	if (j != 4 || strncasecmp("from", string + i, 4)) {
		printf("Error in rule `%s':\n"
		    "second word in rule must be `from'.\n", string);
		return 1;
	}
	i+=j;
	/* Skipping spaces. */
	while( ISSPACE(string[i]) )
		i++;
	/* Senders list. */
	j=0;
	while( !ISSPACE(string[i+j]) )
		j++;
	/* Checking `any' case. */
	if (j == 3 && !strncasecmp("any", string + i, 3)) {
		#ifdef DEBUG
		printf("`from any' rule.\n");
		#endif
		rule->src_all=1;
		rule->src_hash=NULL;
	} else {
		rule->src_all=0;
		if( !(rule->src_hash=get_addresses(string+i,j)) )
			return 1;
	}
	i+=j;
	/* Skipping spaces. */
	while( ISSPACE(string[i]) )
		i++;
	/* 4th word must be `to'. */
	j=0;
	while( !ISSPACE(string[i+j]) )
		j++;
	if (j != 2 || strncasecmp("to", string + i, 2)) {
		printf("Error in rule `%s':\n"
		    "4th word in rule must be `to'.\n", string);
		return 1;
	}
	i+=j;
	/* Skipping spaces. */
	while( ISSPACE(string[i]) )
		i++;
	/* Recipients list. */
	j=0;
	while( !ISSPACE(string[i+j]) && string[i+j] != '\0' )
		j++;
	/* Checking `any' case. */
	if (j == 3 && !strncasecmp("any", string + i, 3)) {
		#ifdef DEBUG
		printf("`to any' rule.\n");
		#endif
		rule->dest_all=1;
		rule->dest_hash=NULL;
	} else {
		rule->dest_all=0;
		if( !(rule->dest_hash=get_addresses(string+i,j)) )
			return 1;
	}

	return 0;
}

HASH_PTR get_addresses(char* string,int len)
/******************************************************************************
 * Parses mail addresses from string `user@domain,user@domain,...' to hash.   *
 ******************************************************************************/
{
	int i,j,n;
	HASH_PTR hash;
	char* addr;

	n=1;
	for(i=0;i<len;i++)
		if( string[i] == ',' )
			n++;

	#ifdef DEBUG
	printf("Extracting %d addresses from `%s' len = %d.\n",n,string,len);
	#endif

	if( !(hash=(HASH_PTR)malloc(sizeof(HASH))) ||
			CreateHash(hash,n) < 0 )
	{
		printf("Unable to allocate/create hash table for mail addresses.\n");
		return NULL;
	}

	j=0;
	for(i=0;i<len;i++)
	{
		if( string[i] == ',' )
		{
			if( !(addr=(char*)calloc(i-j+1,sizeof(char))) )
			{
				printf("Unable to allocate memory.\n");
				return NULL;
			}
			for(n=0;n<i-j;n++)
				addr[n]=string[j+n];
			addr[n]='\0';
			#ifdef DEBUG
			printf("Got address `%s' from %d to %d.\n",addr,j,i-1);
			#endif
			ToHash(hash,addr,NULL);
			free(addr);
			j=i+1;
		}
	}
	if( !(addr=(char*)calloc(i-j+1,sizeof(char))) )
	{
		printf("Unable to allocate memory.\n");
		return NULL;
	}
	for(n=0;n<i-j;n++)
		addr[n]=string[j+n];
	addr[n]='\0';
	#ifdef DEBUG
	printf("Got address `%s'.\n",addr);
	#endif
	ToHash(hash,addr,NULL);
	free(addr);

	return hash;
}
