/*
 * $Id: utils.c 13 2007-10-04 14:54:56Z rea $
 *
 * Copyright 2003-2007 Eygene A. Ryabinkin. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY EYGENE A. RYABINKIN "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EYGENE A. RYABINKIN OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(lint)
static const char vcsid[] = "$Id: utils.c 13 2007-10-04 14:54:56Z rea $";
#endif

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <errno.h>
#include <stdlib.h>
#include "my.h"
#include "hash.h"
#include "stringops.h"

void writeInt(int Sock,int Val)
/******************************************************************************
 * Writes Val into socket Sock, previously translating Val into network order.*
 ******************************************************************************/
{
	Val=htonl(Val);
	write(Sock,&Val,sizeof(Val));
}

void readInt(int Sock,int* Val)
/******************************************************************************
 * Reads Val from socket Sock and translates it from network to machine byte  *
 * order.                                                                     *
 ******************************************************************************/
{
	read(Sock,Val,sizeof(int));
	*Val=ntohl(*Val);
}

int isSocket(char* name)
/******************************************************************************
 * Checks if specified file exists and is a socket.                           *
 ******************************************************************************/
{
	struct stat sb;

	if( stat(name,&sb) < 0 )
	{
		#ifdef DEBUG
		printf("%s,%d: can's stat `%s'.\n",__FILE__,__LINE__,name);
		#endif
		return 0;
	}
	if( S_ISSOCK(sb.st_rdev) )
	{
		#ifdef DEBUG
		printf("%s,%d: `%s' is not a socket.\n",__FILE__,__LINE__,name);
		#endif
		return 0;
	}
	return 1;
}

int SpawnMail(char* mailprog,long* MailPID,char* to)
/******************************************************************************
 * Spawns sendmail, that will send message to specified user and returns      *
 * descriptor to write message body to.                                       *
 ******************************************************************************/
{
	int des[2],pid;

	if( pipe(des) == -1 )
	{
		LogError(-1,"Cannot create pipe!");
		return -1;
	}
	
	while( (pid=fork()) == -1 )
		sleep(2);

	if( pid )
	{
		*MailPID=pid;
		close(des[0]);
		return des[1];
	}
	else
	{
		close(des[1]);
		dup2(des[0],0);
		execl(mailprog,mailprog,to,NULL);
		LogError(-1,"Cannot exec %s: %s.\n",mailprog,strerror(errno));
		exit(-1);
	}
}

void my_printf(int des,char* string,TOKENS_PTR tokens,long num)
/******************************************************************************
 ******************************************************************************/
{
	long i,j,n,len;
	char byte;

	i=j=0;

	while( (byte=string[i]) )
	{
		if( byte == '%' )
		{
			i++;
			byte=string[i];
			/* Flush current string buffer. */
			len=i-j-1;
			write(des,string+j,len);
			j=i;
			/* Check, what we have. */
			for(n=0;n<num;n++)
				if( byte == tokens[n].name )
				{
					if( tokens[n].value )
						write(des,tokens[n].value,strlen(tokens[n].value));
					else
						write(des,"(null)",6);
					i++; j++;
					break;
				}
		}
		else
			i++;
	}
	len=i-j;
	write(des,string+j,len);
	return;
}

struct list {
	char* name;
	struct list * next;
};

#define BUFLEN 1024

int ReadUsers(FILE* fp,HASH_PTR hash)
{
	struct list * root=NULL;
	struct list * current;
	struct list * prev=NULL;
	char buffer[BUFLEN];
	long num, len;

	num=0;
	fgets(buffer,BUFLEN,fp);
	while( !feof(fp) )
	{
		chomp(buffer);
		len = strlen(buffer) + 1;
		if (!(current=(struct list*)malloc(sizeof(struct list))) ||
		    !(current->name=(char*)malloc(len*sizeof(char))))
			return -1;
		if( !root )
			root=current;
		strlcpy(current->name, buffer, len);
		current->next=NULL;
		if( prev )
			prev->next=current;
		prev=current;
		num++;
		fgets(buffer,BUFLEN,fp);
	}
	
	if( CreateHash(hash,num) == -1 )
		return -1;

	current=root;

	while( current )
	{
		ToHash(hash,current->name,NULL);
		prev=current;
		current=current->next;
		free(prev->name);
		free(prev);
	}
	return 0;
}

void StripSymbols(char* string,char* symb,long size)
/******************************************************************************
 * Function strips from 'string' all symbols, contained in 'symb'. 'size' is  *
 * the length of 'symb'.                                                      *
 ******************************************************************************/
{
	long i,j,n;

	if( !string )
		return;

	for(n=0;n<size;n++)
		if( !symb[n] )
		{
			LogError(-1,"StripSymbols(): '\\0' detected at position %ld in stripping symbols set.",n);
			return;
		}

	i=j=0;
	while( string[j] )
	{
		for(n=0;n<size;n++)
			if( symb[n] == string[j] )
				j++;
		string[i]=string[j];
		if( string[j] == '\0' )
			break;
		i++; j++;
	}

	return;
}
